/*
JunioSonic Peelout
	by Lach
*/

local SKIN = "juniosonic" // skin to run script for
local CHARGE_SPEED = FRACUNIT // peelout speed charged per tic

local function valid(mo)
	return mo and mo.valid
end

local function CanPeelout(player)
	local var = player.pflags & (PF_SLIDING|PF_SPINNING)
		or player.junio_dontpeel
		or player.exiting
		or P_PlayerInPain(player)
		or player.powers[pw_carry]
		or player.charability2 ~= skins[SKIN].ability2
		or player.playerstate == PST_DEAD
		or player.pflags & PF_STASIS 
		or player.pflags & PF_FULLSTASIS
		or player.mo.state == S_PLAY_SPRING
	return not var
end

local function DoSpinDashDust(player, speed) // From the source code!
	local mo = player.mo
	local prandom = {}
	for i = 0, (leveltime%7) >> 1 // 1, 2, 3 or 4 particles
		local particle = P_SpawnMobjFromMobj(mo, 0, 0, 0, MT_SPINDUST)

		if mo.eflags & (MFE_TOUCHWATER|MFE_UNDERWATER) // overrides fire version
			particle.state = S_SPINDUST_BUBBLE1
		elseif player.powers[pw_shield] == SH_ELEMENTAL
			particle.state = S_SPINDUST_FIRE1
		end

		particle.target = mo
		particle.destscale = (mo.destscale << 1)/3
		P_SetScale(particle, particle.destscale)
		if mo.eflags & MFE_VERTICALFLIP // readjust z position if needed
			particle.z = mo.z + mo.height - particle.height
		end
		prandom[0] = P_RandomFixed()<<2 // P_RandomByte()<<10
		prandom[1] = P_RandomRange(-30, 30) // P_RandomRange(-ANG30/FRACUNIT, ANG30/FRACUNIT)*FRACUNIT
		prandom[2] = P_RandomFixed()<<3 // P_RandomByte()<<11
		P_SetObjectMomZ(particle, speed/50 + prandom[0], false)
		P_InstaThrust(particle,
				player.drawangle + prandom[1]*ANG1,
				-FixedMul(speed/12 + FRACUNIT + prandom[2], mo.scale))
		P_TryMove(particle, particle.x + particle.momx, particle.y + particle.momy, true)
	end
end

// Activate peelout

addHook("SpinSpecial", function(player)
	local mo = player.mo
	
	if not mo
	or mo.skin ~= SKIN
	or player.pflags & PF_USEDOWN
	or not CanPeelout(player)
	or player.speed >= 5*mo.scale
	or mo.momz
	or not P_IsObjectOnGround(mo)
	or (mo.standingslope and not (mo.standingslope.flags & SL_NOPHYSICS) and abs(mo.standingslope.zdelta) >= (FRACUNIT >> 1)) // not on steep slopes
		return
	end
	
	mo.momx = player.cmomx
	mo.momy = player.cmomy
	
	S_StartSound(mo, sfx_plchrg)
	
	player.junio_peelout = player.mindash
	
	return true
end)

// Peelout handling

addHook("ThinkFrame", do
	for player in players.iterate
		local mo = player.mo
		
		if not mo
		or mo.skin ~= SKIN
		or player.playerstate
		or player.powers[pw_carry]
			if player.junio_peelout
				player.junio_peelout = nil
			end
			continue
		end
		
		if player.panim == PA_RUN
			mo.tics = min($, 1)
		end
		
		if player.juniodashmode >= 3*TICRATE
		and leveltime % 3 == 0
		and P_IsObjectOnGround(mo)
		and mo.eflags & MFE_TOUCHWATER
		and not (P_PlayerTouchingSectorSpecial(player, 1, 3))
			for i = -1, 1, 2
				local ang = i * ANG10
				local factor = FixedMul(4*mo.scale, cos(ang))
				local x = P_ReturnThrustX(mo, player.drawangle + ANGLE_180 + ang, 24*mo.scale)
				local y = P_ReturnThrustY(mo, player.drawangle + ANGLE_180 + ang, 24*mo.scale)
				local dust = P_SpawnMobjFromMobj(mo, x, y, 0, MT_THOK)
				dust.scale = 2*$/3
				dust.angle = R_PointToAngle2(dust.x, dust.y, mo.x, mo.y)
				dust.state = S_KICKDUST
				dust.momx = FixedMul(factor, cos(dust.angle - i*ANGLE_90))
				dust.momy = FixedMul(factor, sin(dust.angle - i*ANGLE_90))
			end
		end
		if player.junio_peelout ~= nil // The player is charging a peelout
			if not CanPeelout(player) // cancel the peelout if you shouldn't be charging one
				player.junio_peelout = nil
				if not (P_PlayerInPain(player))
				and not (mo.state == S_PLAY_SPRING)
					mo.state = S_PLAY_WALK
				end
			elseif player.pflags & PF_USEDOWN // gotta be holding spin to charge a peelout
				if player.cmd.buttons & BT_JUMP // if jump is pressed or held, switch to spindash instead
					player.pflags = $ | PF_JUMPDOWN | PF_STARTDASH | PF_SPINNING
					player.dashspeed = min(player.junio_peelout, player.maxdash)
					player.junio_peelout = nil
					mo.state = S_PLAY_SPINDASH
					S_StartSound(mo, sfx_spndsh)
				else // charge the peelout!
					local mincharge = player.mindash
					local maxcharge = skins[SKIN].normalspeed * 2
					local currentcharge
					local chargelevel
					
					if player.junio_peelout < maxcharge // increment the peelout charge
						player.junio_peelout = $ + CHARGE_SPEED
						if player.junio_peelout >= maxcharge
							S_StartSound(mo, sfx_cdfm40)
						end
					end
					
					currentcharge = player.junio_peelout
					chargelevel = 8 * (currentcharge - mincharge) / ((maxcharge - mincharge) or 1)
					
					if chargelevel == 8 // change animation based on charge level
						mo.state = S_PLAY_DASH
					elseif chargelevel >= 6
						mo.state = S_PLAY_RUN
					elseif chargelevel >= 4
						if mo.state ~= S_PLAY_RUN
							mo.state = S_PLAY_RUN
						end
					else
						if mo.state ~= S_PLAY_WALK
							mo.state = S_PLAY_WALK
						end
						mo.tics = min($, 3 - (chargelevel >> 1))
					end
					
					mo.angle = player.cmd.angleturn << 16 // set angle and remove control
					player.drawangle = mo.angle
					player.pflags = $ | PF_FULLSTASIS
					mo.reactiontime = 1
					
					DoSpinDashDust(player, currentcharge)
				end
			else // release the peelout!
				if P_IsObjectOnGround(mo)
					local mincharge = player.mindash
					local normalspeed = skins[SKIN].normalspeed
					local currentcharge = player.junio_peelout
					local jumpfactorboost = FixedDiv(max(0, currentcharge - normalspeed - mincharge), normalspeed - mincharge)
					player.juniodashmode = 3*TICRATE		 
					player.normalspeed = max($, currentcharge)
					player.jumpfactor = skins[SKIN].jumpfactor + FixedMul(jumpfactorboost, skins[SKIN].jumpfactor >> 2)
					P_InstaThrust(mo, mo.angle, FixedMul(currentcharge, mo.scale))
					S_StartSound(mo, sfx_plrles)
				end
				player.junio_peelout = nil
			end
		end
	end
end)

// Sonic + Dash Mode 
// Created by Wolfy to teach Iceman404 shit about lua
// Requested, revamped, and continued by Iceman404 
// Guidance from HellHawkX, Zipper, katsy, RedEnchilada and Sryder13.
// toaster understands how scale works, makes Wolfs sad by threatening to remove this lump, and catches him from doing terrible things like overriding all players' attributes with their default every tic
// PLEASE VIEW WITH NOTEPAD++ FOR FORMATTING SANITY.

--Ported by MotdSpork for junioSonic because fuck the default dashmode's flashing. It really does not fit junioSonic...like at all.

assert(VERSION ~= nil, "script does not support game version")

//PREPARATION
addHook("MobjThinker", function(mobj) 	// This is a MobjThinker hook, it runs for every tic the mobj is on the map.
	local player = mobj.player 			// A little alias I do to make these similar to ThinkFrame hooks.
	
	if not (player and player.valid) then return end
	if (player.powers[pw_carry] == CR_NIGHTSMODE) return end

	if (mobj.skin == "juniosonic") then
		if not player.juniodashmode then 
			player.juniodashmode = 0 			//This makes sure the variable starts at 0, and not nil. Otherwise, you can't perform arithmetic on it.
			player.juniodashmodeflag = false
		end
		
		player.defspeed = skins["juniosonic"].normalspeed 	// Default normalspeed.
		player.maxtop = skins[mobj.skin].normalspeed * 80/36
		

		//HOW DASH MODE COUNTER WORKS
		local speed = FixedDiv(player.speed, mobj.scale) // toast is fixing all the everythings.
		if speed >= (player.defspeed - 5*FRACUNIT) or (player.pflags & PF_STARTDASH) then
			//if not (player.powers[pw_super]) then
				player.juniodashmode = $1 + 1 	// Counter. Adds 1 to dash mode per tic in top speed.
				if player.juniodashmode == 3*TICRATE				// This isn't in the ">=" equation because it'd cause the sound to play infinitely.
					S_StartSound (mobj, sfx_s3ka2) 	// If the player enters juniodashmode, play this sound on the the tic it starts.
				end
			//else
				//player.juniodashmode = 0
			//end
		elseif not (player.pflags & PF_SPINNING)
		and not (player.pflags & PF_JUMPED) then
			if player.juniodashmode > 0 then
				player.juniodashmode = $1 - 3	// Rather than lose it all, it gently counts back down!
			elseif player.juniodashmode < 0 then 
				player.juniodashmode = 0
			end
		end	
	
		//PARAMETERS FOR DASH MODE TO EVEN HAPPEN
		if player.juniodashmode >= 3*TICRATE and P_IsObjectOnGround(mobj) and player.playerstate == PST_LIVE then  // Dash Mode can continue counting in the air, but will only activate on floor touch.
			player.juniodashmodeflag = true		
			if not (player.pflags & PF_SPINNING)
			and not (player.pflags & PF_JUMPED)
				DoSpinDashDust(player, player.juniodashmode/4)
				player.mo.state = S_PLAY_DASH
			end
		end
		
		if player.juniodashmode < 3*TICRATE
		and player.rev == 0 then			// Exits Dash Mode if you drop below speed/dash counter tics. Not in the above block so it doesn't keep disabling in midair.
			player.normalspeed = player.defspeed 	// Reset to default if not capable of entering dash mode.
			player.jumpfactor = 1*FRACUNIT
			player.juniodashmodeflag = false
			if player.mo.state == S_PLAY_DASH
			and player.junio_peelout == nil
				player.mo.state = S_PLAY_RUN
			end
		end
		
		//WHEN PARAMETERS ARE MET, REWARD THE DASH MODE EFFECTS
		if player.juniodashmodeflag
		and player.charging == false then
			if player.normalspeed < player.maxtop then  	// If the player is not currently at 50 normalspeed in dash mode, add speed each tic
				player.normalspeed = $1 + 1*FRACUNIT/5 			// Enter Dash Mode smoothly.
				if player.jumpfactor < 5*FRACUNIT/4 then
					player.jumpfactor = $1 + 1*FRACUNIT/300 	// Boosts his jumpheight. Remember fractions instead of decimals. "1.5*FRACUNIT = 3*FRACUNIT/2"				
				end
			end		
		end
		
		//COSMETIC STUPIDITY!
		if player.juniodashmode > 108 then	//Dash Mode will go down a tic a bit above activation, this makes dust spawn every other tic.
			player.juniodashmode = 107
		end


		if player.juniodashmode >= 3*TICRATE then
			local ghost = P_SpawnGhostMobj(mobj) 	// Spawns afterimages
			ghost.fuse = 2								// Makes the images fade quickly
			player.charflags = $1|SF_RUNONWATER
		else
			player.charflags = $1 & ~SF_RUNONWATER
		end
	end
end, MT_PLAYER)